@@ -11,7 +11,6 @@ function(sgp_object,
 
 	TARGET_STATUS_INITIAL <- VALID_CASE <- ID <- CONTENT_AREA <- YEAR <- NULL
 
-
 	### Define variables
 
 	if (target.level=="CATCH_UP_KEEP_UP") level.to.get <- which.max(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]]=="Proficient")-1
@@ -42,7 +41,7 @@ function(sgp_object,
 		cols.to.get.names <- names(sgp_object@SGP[["SGProjections"]][[i]])[
 			grep(paste("LEVEL_", level.to.get, sep=""), names(sgp_object@SGP[["SGProjections"]][[i]]))]
 		num.years.to.get <- min(max.sgp.target.years.forward, length(cols.to.get.names))
-		cols.to.get.names <- cols.to.get.names[as.integer(sapply(sapply(cols.to.get.names, strsplit, "_"), tail, 1) ) <= num.years.to.get]
+		cols.to.get.names <- cols.to.get.names[as.integer(sapply(strsplit(sapply(sapply(cols.to.get.names, strsplit, "_YEAR_"), tail, 1), "_"), head, 1)) <= num.years.to.get]
 		if (target.type %in% c("sgp.projections.lagged", "sgp.projections.lagged.baseline")) cols.to.get.names <- c("ACHIEVEMENT_LEVEL_PRIOR", cols.to.get.names)
 		cols.to.get <- sapply(cols.to.get.names, function(x) which(x==names(sgp_object@SGP[["SGProjections"]][[i]])))
 		tmp.list[[i]] <- data.table(
@@ -71,10 +70,10 @@ function(sgp_object,
 	tmp_object_2 <- tmp_object_1[, eval(jExpression), by=list(ID, CONTENT_AREA, YEAR, VALID_CASE)]
 
 	if (target.type %in% c("sgp.projections.baseline", "sgp.projections.lagged.baseline")) baseline.label <- "_BASELINE" else baseline.label <- NULL
-	if (target.type %in% c("sgp.projections", "sgp.projections.baseline")) projection.label <- "_CURRENT_YEAR_PROJECTION" else projection.label <- NULL
+	if (target.type %in% c("sgp.projections", "sgp.projections.baseline")) projection.label <- "_CURRENT" else projection.label <- NULL
 	if (target.level=="MOVE_UP_STAY_UP") target.level.label <- "_MOVE_UP_STAY_UP" else target.level.label <- NULL
 
-	setnames(tmp_object_2, "V1", paste("SGP_TARGET", baseline.label, projection.label, target.level.label, "_",  max.sgp.target.years.forward.label, "_YEAR", sep=""))
+	setnames(tmp_object_2, "V1", paste("SGP_TARGET", baseline.label, target.level.label, "_",  max.sgp.target.years.forward.label, "_YEAR", projection.label, sep=""))
 
 	if (target.type %in% c("sgp.projections.lagged", "sgp.projections.lagged.baseline") & return.lagged.status) {
 		tmp_object_2[,c("ACHIEVEMENT_LEVEL_PRIOR", grep("STATUS_INITIAL", names(tmp_object_1), value=TRUE)) := 
@@ -83,5 +82,4 @@ function(sgp_object,
 
 	setkeyv(tmp_object_2, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
 	return(tmp_object_2)
-
 } ### END getTargetSGP
